import { TestSession as SSession } from "@peerbit/libp2p-test-utils";
import { waitForPeers as waitForPeersStreams } from "@peerbit/stream";
import { DirectBlock } from "@peerbit/blocks";
import { DirectSub } from "@peerbit/pubsub";
import { Peerbit } from "peerbit";
import { mplex } from "@libp2p/mplex";
import { DefaultKeychain } from "@peerbit/keychain";
export class TestSession {
    session;
    _peers;
    constructor(session, peers) {
        this.session = session;
        this._peers = peers;
    }
    get peers() {
        return this._peers;
    }
    async connect(groups) {
        await this.session.connect(groups?.map((x) => x.map((y) => y)));
        return;
    }
    async stop() {
        await Promise.all(this._peers.map((x) => x.stop()));
        await Promise.all(this._peers.map((x) => x.libp2p.stop())); // beacuse we initialize libp2p externally, we have to close externally
    }
    static async connected(n, options) {
        const session = await TestSession.disconnected(n, options);
        await session.connect();
        // TODO types
        await waitForPeersStreams(...session.peers.map((x) => x.services.blocks));
        return session;
    }
    static async disconnected(n, options) {
        const m = (o) => {
            return {
                ...o?.libp2p,
                services: {
                    blocks: (c) => new DirectBlock(c),
                    pubsub: (c) => new DirectSub(c, { canRelayMessage: true }),
                    keychain: (c) => new DefaultKeychain(),
                    ...o?.libp2p?.services
                },
                streamMuxers: [mplex({ disconnectThreshold: 10 })]
            };
        };
        let optionsWithServices = Array.isArray(options)
            ? options.map(m)
            : m(options);
        const session = await SSession.disconnected(n, optionsWithServices);
        return new TestSession(session, (await Promise.all(session.peers.map((x, ix) => Array.isArray(options)
            ? Peerbit.create({ libp2p: x, directory: options[ix]?.directory })
            : Peerbit.create({ libp2p: x, directory: options?.directory })))));
    }
}
//# sourceMappingURL=session.js.map